@@ -14,22 +14,6 @@ use PAGI::Util::AsyncFile;
 
 
 use constant FILE_CHUNK_SIZE => 65536;  # 64KB chunks for file streaming
-use constant DEFAULT_SENDFILE_TIMEOUT => 30;  # Default timeout for sendfile socket writability (seconds)
-
-# Try to load Sys::Sendfile for zero-copy file transfers
-# Falls back to chunked reads if not available
-my $HAS_SENDFILE;
-BEGIN {
-    eval {
-        require Sys::Sendfile;
-        Sys::Sendfile->import('sendfile');
-        $HAS_SENDFILE = 1;
-    };
-    $HAS_SENDFILE //= 0;
-}
-
-# Class method to check if sendfile is available at the system level
-sub has_sendfile { return $HAS_SENDFILE }
 
 # =============================================================================
 # Header Validation (CRLF Injection Prevention)
@@ -106,17 +90,6 @@ connection. It handles:
 
 =back
 
-=head2 Sendfile Timeout Configuration
-
-When using sendfile() for large file transfers, the socket buffer may fill
-up faster than the client can drain it. The C<sendfile_timeout> parameter
-controls how long to wait for the socket to become writable before giving up.
-
-B<Default:> 30 seconds
-
-Configure via the C<sendfile_timeout> parameter in L<PAGI::Server> or
-directly in the Connection constructor.
-
 =cut
 
 sub new {
@@ -138,9 +111,7 @@ sub new {
         access_log        => $args{access_log},     # Filehandle for access logging
         max_receive_queue => $args{max_receive_queue} // 1000,  # Max WebSocket receive queue size
         max_ws_frame_size => $args{max_ws_frame_size} // 65536,  # Max WebSocket frame size in bytes
-        disable_sendfile  => $args{disable_sendfile} // 0,  # Disable sendfile even if available
-        sync_file_threshold => $args{sync_file_threshold} // 65536,  # Threshold for sync file reads (0=always async)
-        sendfile_timeout  => $args{sendfile_timeout} // DEFAULT_SENDFILE_TIMEOUT,  # Timeout for sendfile socket writability (seconds)
+        sync_file_threshold => $args{sync_file_threshold} // 65536,  # Threshold for sync file reads (default 64KB)
         tls_info      => undef,  # Populated on first request if TLS
         buffer        => '',
         closed        => 0,
@@ -2038,73 +2009,9 @@ sub _process_websocket_frames {
     }
 }
 
-# Check if we can use sendfile() for this connection
-sub _can_use_sendfile {
-    my ($self, $chunked) = @_;
-
-    return 0 unless $HAS_SENDFILE;
-    return 0 if $self->{disable_sendfile};  # Explicitly disabled
-    return 0 if $self->{tls_enabled};       # Can't sendfile over TLS
-    return 0 if $chunked;                   # Can't sendfile with chunked encoding (need headers)
-    return 1;
-}
-
-# Wait for socket to become writable (non-blocking, with timeout)
-# Used by sendfile loop when socket buffer is full (EAGAIN condition)
-# Returns a Future that resolves when socket is writable or fails on timeout/error
-use constant DEFAULT_SENDFILE_TIMEOUT => 30;  # seconds
-
-sub _wait_for_writable {
-    my ($self, $socket) = @_;
-
-    my $loop = $self->{server} ? $self->{server}->loop : undef;
-    die "No event loop available for _wait_for_writable" unless $loop;
-
-    # Check if connection is already closed
-    return Future->fail('Connection closed') if $self->{closed};
-
-    my $write_ready_f = $loop->new_future;
-    my $watching = 0;
-
-    # Set up write readiness watch
-    $loop->watch_io(
-        handle => $socket,
-        on_write_ready => sub {
-            return unless $watching;  # Guard against double-fire
-            $watching = 0;
-            $loop->unwatch_io(handle => $socket, on_write_ready => 1);
-            $write_ready_f->done unless $write_ready_f->is_ready;
-        },
-    );
-    $watching = 1;
-
-    # Set up timeout to prevent infinite wait
-    my $timeout = $self->{sendfile_timeout};
-    my $timeout_f = $loop->delay_future(after => $timeout)->then(sub {
-        Future->fail("sendfile write timeout after $timeout seconds");
-    });
-
-    # Race between write ready and timeout
-    my $result_f = Future->wait_any($write_ready_f, $timeout_f);
-
-    # Ensure cleanup on completion (success, failure, or cancellation)
-    $result_f->on_ready(sub {
-        if ($watching) {
-            $watching = 0;
-            eval { $loop->unwatch_io(handle => $socket, on_write_ready => 1) };
-        }
-        # Cancel whichever future didn't complete
-        $write_ready_f->cancel unless $write_ready_f->is_ready;
-        $timeout_f->cancel unless $timeout_f->is_ready;
-    });
-
-    return $result_f;
-}
-
 # Async file response - prioritizes speed based on file size:
 #   1. Small files (<=64KB): direct in-process read (fastest for small files)
-#   2. Large files with sendfile: zero-copy kernel transfer (fastest for large files)
-#   3. Large files without sendfile: worker pool (non-blocking fallback)
+#   2. Large files: async chunked reads via worker pool (non-blocking)
 async sub _send_file_response {
     my ($self, $file, $offset, $length, $chunked) = @_;
 
@@ -2113,9 +2020,11 @@ async sub _send_file_response {
     die "Cannot stat file $file: $!" unless defined $file_size;
     $length //= $file_size - $offset;
 
+    my $stream = $self->{stream};
+
     if ($self->{sync_file_threshold} > 0 && $length <= $self->{sync_file_threshold}) {
-        # Small file fast path: read directly in-process (no syscall/IPC overhead)
-        # For files <= 64KB, a simple read() beats sendfile() due to syscall overhead
+        # Small file fast path: read directly in-process
+        # For files <= 64KB, a simple read() is fast and avoids async overhead
         open my $fh, '<:raw', $file or die "Cannot open file $file: $!";
         seek($fh, $offset, 0) if $offset;
         my $bytes_read = read($fh, my $data, $length);
@@ -2123,7 +2032,6 @@ async sub _send_file_response {
 
         die "Failed to read file $file: $!" unless defined $bytes_read;
 
-        my $stream = $self->{stream};
         if ($chunked) {
             my $len = sprintf("%x", length($data));
             $stream->write("$len\r\n$data\r\n");
@@ -2133,68 +2041,15 @@ async sub _send_file_response {
             $stream->write($data);
         }
     }
-    elsif ($self->_can_use_sendfile($chunked)) {
-        # Large file sendfile path: zero-copy kernel transfer
-        # Ensure headers are flushed before sendfile (sendfile bypasses IO::Async buffer)
-        await $self->{stream}->write('');
-
-        open my $fh, '<:raw', $file or die "Cannot open file $file: $!";
-        my $socket = $self->{stream}->write_handle;
-
-        # sendfile may not send all bytes in one call, loop until done
-        # Sys::Sendfile expects filehandles, not file descriptor numbers
-        # On non-blocking sockets, sendfile returns 0 when buffer is full (EAGAIN)
-        my $sent = 0;
-        my $sendfile_error;
-
-        while ($sent < $length) {
-            # Check if connection was closed during transfer
-            if ($self->{closed}) {
-                $sendfile_error = "Connection closed during sendfile";
-                last;
-            }
-
-            my $to_send = $length - $sent;
-            my $current_offset = $offset + $sent;
-            my $result = sendfile($socket, $fh, $to_send, $current_offset);
-
-            if (!defined $result || $result < 0) {
-                $sendfile_error = "sendfile failed: $!";
-                last;
-            }
-
-            $sent += $result;
-
-            # Handle EAGAIN: sendfile returned 0 but we haven't sent everything
-            # This happens on non-blocking sockets when the send buffer is full
-            # Wait for socket to become writable before retrying (with timeout)
-            if ($result == 0 && $sent < $length) {
-                my $wait_ok = eval { await $self->_wait_for_writable($socket); 1 };
-                if (!$wait_ok) {
-                    $sendfile_error = $@ || "Failed waiting for socket writability";
-                    last;
-                }
-                next;
-            }
-
-            last if $result == 0;  # True EOF (shouldn't happen if length is correct)
-        }
-
-        close $fh;
-        die $sendfile_error if $sendfile_error;
-    }
     else {
-        # Worker pool path - non-blocking chunked reads for large files
-        # Used when sendfile unavailable (TLS, chunked encoding, no Sys::Sendfile)
+        # Large file path: async chunked reads via worker pool
         my $loop = $self->{server} ? $self->{server}->loop : undef;
         die "No event loop available for async file I/O" unless $loop;
 
-        my $stream = $self->{stream};
-
         await PAGI::Util::AsyncFile->read_file_chunked(
             $loop, $file,
-            sub  {
-        my ($chunk) = @_;
+            sub {
+                my ($chunk) = @_;
                 if ($chunked) {
                     my $len = sprintf("%x", length($chunk));
                     $stream->write("$len\r\n$chunk\r\n");
